@@ -19,7 +19,6 @@
  *****************************************************************************/
 
 #include "filemetadataprovider.h"
-#include "indexeddataretriever.h"
 #include "filefetchjob.h"
 
 #include <KFileMetaData/PropertyInfo>
@@ -74,7 +73,7 @@ namespace {
             QSet<QString> s1 = v1.toStringList().toSet();
             QSet<QString> s2 = v2.toStringList().toSet();
 
-            return QVariant(s1.intersect(s2).toList());
+            return QVariant(s1.intersect(s2).values());
         }
 
         if (v1 == v2) {
@@ -137,8 +136,7 @@ void FileMetaDataProvider::slotFileFetchFinished(KJob* job)
     if (files.size() > 1) {
         insertCommonData(files);
     } else {
-        m_data = files.first();
-        insertSingleFileBasicData();
+        m_data = unite(m_data, files.first());
     }
     m_readOnly = !fetchJob->canEditAll();
 
@@ -146,15 +144,6 @@ void FileMetaDataProvider::slotFileFetchFinished(KJob* job)
     emit loadingFinished();
 }
 
-void FileMetaDataProvider::slotLoadingFinished(KJob* job)
-{
-    IndexedDataRetriever* ret = dynamic_cast<IndexedDataRetriever*>(job);
-    m_data = unite(m_data, ret->data());
-    m_readOnly = !ret->canEdit();
-
-    emit loadingFinished();
-}
-
 void FileMetaDataProvider::insertSingleFileBasicData()
 {
     // TODO: Handle case if remote URLs are used properly. isDir() does
@@ -164,12 +153,30 @@ void FileMetaDataProvider::insertSingleFileBasicData()
       const KFileItem& item = m_fileItems.first();
 
       if (item.isDir()) {
-          const int count = subDirectoriesCount(item.url().path());
-          if (count == -1) {
+          bool isSizeUnknown = !item.isLocalFile();
+          if (!isSizeUnknown) {
+              const QPair<int, int> counts = subDirectoriesCount(item.url().path());
+              const int count = counts.first;
+              isSizeUnknown = count == -1;
+              if (!isSizeUnknown) {
+                  QString itemCountString = i18ncp("@item:intable", "%1 item", "%1 items", count);
+                  m_data.insert(QStringLiteral("kfileitem#size"), itemCountString);
+
+                  const int hiddenCount = counts.second;
+                  if (hiddenCount > 0) {
+                      // add hidden items count
+                      QString hiddenCountString = i18ncp("@item:intable", "%1 item", "%1 items", hiddenCount);
+                      m_data.insert(QStringLiteral("kfileitem#hiddenItems"), hiddenCountString);
+                  }
+              }
+          }
+          else if (item.entry().contains(KIO::UDSEntry::UDS_SIZE)) {
+              isSizeUnknown = false;
+              KFormat format;
+              m_data.insert(QStringLiteral("kfileitem#size"), format.formatByteSize(item.size()));
+          }
+          if (isSizeUnknown) {
               m_data.insert(QStringLiteral("kfileitem#size"), i18nc("unknown file size", "Unknown"));
-          } else {
-              const QString itemCountString = i18ncp("@item:intable", "%1 item", "%1 items", count);
-              m_data.insert(QStringLiteral("kfileitem#size"), itemCountString);
           }
       } else {
           KFormat format;
@@ -177,6 +184,9 @@ void FileMetaDataProvider::insertSingleFileBasicData()
       }
 
       m_data.insert(QStringLiteral("kfileitem#type"), item.mimeComment());
+      if (item.isLink()) {
+          m_data.insert(QStringLiteral("kfileitem#linkDest"), item.linkDest());
+      }
       m_data.insert(QStringLiteral("kfileitem#modified"), item.time(KFileItem::ModificationTime));
       QDateTime creationTime = item.time(KFileItem::CreationTime);
       if (creationTime.isValid()) {
@@ -203,10 +213,28 @@ void FileMetaDataProvider::insertFilesListBasicData()
 
     if (allDirectories) {
         int count = 0;
+        int hiddenCount = 0;
+        bool isSizeKnown = true;
         for (const KFileItem& item : qAsConst(m_fileItems)) {
-            count += subDirectoriesCount(item.url().path());
+            isSizeKnown = item.isLocalFile();
+            if (!isSizeKnown) {
+                return;
+            }
+            const QPair<int, int> counts = subDirectoriesCount(item.url().path());
+            const int subcount = counts.first;
+            isSizeKnown = subcount != -1;
+            if (!isSizeKnown) {
+                return;
+            }
+            count += subcount;
+            hiddenCount += counts.second;
+        }
+        QString itemCountString = i18ncp("@item:intable", "%1 item", "%1 items", count);
+        if (hiddenCount > 0) {
+            // add hidden items count
+            QString hiddenCountString = i18ncp("@item:intable", "%1 item", "%1 items", hiddenCount);
+            m_data.insert(QStringLiteral("kfileitem#hiddenItems"), hiddenCountString);
         }
-        const QString itemCountString = i18ncp("@item:intable", "%1 item", "%1 items", count);
         m_data.insert(QStringLiteral("kfileitem#totalSize"), itemCountString);
 
     } else {
@@ -298,34 +326,33 @@ void FileMetaDataProvider::setFileItem()
     //   * Not Indexed
     //   * Indexed
     //
+    insertSingleFileBasicData();
     const QUrl url = m_fileItems.first().targetUrl();
     if (!url.isLocalFile()) {
         // FIXME - are extended attributes supported for remote files?
         m_readOnly = true;
-        insertSingleFileBasicData();
         emit loadingFinished();
         return;
     }
 
-    // Not indexed or only basic file indexing (no content)
     const QString filePath = url.toLocalFile();
+    FileFetchJob* job;
+
+    // Not indexed or only basic file indexing (no content)
     if (!m_config.fileIndexingEnabled() || !m_config.shouldBeIndexed(filePath)
             || m_config.onlyBasicIndexing()) {
         m_realTimeIndexing = true;
 
-        insertSingleFileBasicData();
-        insertEditableData();
-
-        IndexedDataRetriever *ret = new IndexedDataRetriever(filePath, this);
-        connect(ret, &IndexedDataRetriever::finished, this, &FileMetaDataProvider::slotLoadingFinished);
-        ret->start();
+        job = new FileFetchJob(QStringList{filePath}, true,
+            FileFetchJob::UseRealtimeIndexing::Only, this);
 
     // Fully indexed by Baloo
     } else {
-        FileFetchJob* job = new FileFetchJob(QStringList{filePath}, true, this);
-        connect(job, &FileFetchJob::finished, this, &FileMetaDataProvider::slotFileFetchFinished);
-        job->start();
+        job = new FileFetchJob(QStringList{filePath}, true,
+            FileFetchJob::UseRealtimeIndexing::Fallback, this);
     }
+    connect(job, &FileFetchJob::finished, this, &FileMetaDataProvider::slotFileFetchFinished);
+    job->start();
 }
 
 void FileMetaDataProvider::setFileItems()
@@ -350,7 +377,8 @@ void FileMetaDataProvider::setFileItems()
         // Editing only if all URLs are local
         bool canEdit = (urls.size() == m_fileItems.size());
 
-        FileFetchJob* job = new FileFetchJob(urls, canEdit, this);
+        FileFetchJob* job = new FileFetchJob(urls, canEdit,
+            FileFetchJob::UseRealtimeIndexing::Disabled, this);
         connect(job, &FileFetchJob::finished, this, &FileMetaDataProvider::slotFileFetchFinished);
         job->start();
 
@@ -390,7 +418,9 @@ QString FileMetaDataProvider::label(const QString& metaDataLabel) const
         { QStringLiteral("kfileitem#size"), i18nc("@label", "Size") },
         { QStringLiteral("kfileitem#tags"), i18nc("@label", "Tags") },
         { QStringLiteral("kfileitem#totalSize"), i18nc("@label", "Total Size") },
+        { QStringLiteral("kfileitem#hiddenItems"), i18nc("@label", "Hidden items") },
         { QStringLiteral("kfileitem#type"), i18nc("@label", "Type") },
+        { QStringLiteral("kfileitem#linkDest"), i18nc("@label", "Link to") },
         { QStringLiteral("tags"), i18nc("@label", "Tags") },
         { QStringLiteral("rating"), i18nc("@label", "Rating") },
         { QStringLiteral("userComment"), i18nc("@label", "Comment") },
@@ -411,14 +441,16 @@ QString FileMetaDataProvider::group(const QString& label) const
 
         // KFileItem Data
         { QStringLiteral("kfileitem#type"), QStringLiteral("0FileItemA") },
-        { QStringLiteral("kfileitem#size"), QStringLiteral("0FileItemB") },
-        { QStringLiteral("kfileitem#totalSize"), QStringLiteral("0FileItemB") },
-        { QStringLiteral("kfileitem#modified"), QStringLiteral("0FileItemC") },
-        { QStringLiteral("kfileitem#accessed"), QStringLiteral("0FileItemD") },
-        { QStringLiteral("kfileitem#created"), QStringLiteral("0FileItemE") },
-        { QStringLiteral("kfileitem#owner"), QStringLiteral("0FileItemF") },
-        { QStringLiteral("kfileitem#group"), QStringLiteral("0FileItemG") },
-        { QStringLiteral("kfileitem#permissions"), QStringLiteral("0FileItemH") },
+        { QStringLiteral("kfileitem#linkDest"), QStringLiteral("0FileItemB") },
+        { QStringLiteral("kfileitem#size"), QStringLiteral("0FileItemC") },
+        { QStringLiteral("kfileitem#totalSize"), QStringLiteral("0FileItemC") },
+        { QStringLiteral("kfileitem#hiddenItems"), QStringLiteral("0FileItemD") },
+        { QStringLiteral("kfileitem#modified"), QStringLiteral("0FileItemE") },
+        { QStringLiteral("kfileitem#accessed"), QStringLiteral("0FileItemF") },
+        { QStringLiteral("kfileitem#created"), QStringLiteral("0FileItemG") },
+        { QStringLiteral("kfileitem#owner"), QStringLiteral("0FileItemH") },
+        { QStringLiteral("kfileitem#group"), QStringLiteral("0FileItemI") },
+        { QStringLiteral("kfileitem#permissions"), QStringLiteral("0FileItemJ") },
 
         // Editable Data
         { QStringLiteral("tags"), QStringLiteral("1EditableDataA") },
@@ -486,19 +518,23 @@ QVariantMap FileMetaDataProvider::data() const
     return m_data;
 }
 
-int FileMetaDataProvider::subDirectoriesCount(const QString& path)
+QPair<int, int> FileMetaDataProvider::subDirectoriesCount(const QString& path)
 {
 #ifdef Q_OS_WIN
     QDir dir(path);
-    return dir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot|QDir::System).count();
+    int count = dir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot|QDir::System).count();
+    int hiddenCount = dir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot|QDir::System|QDir::Hidden).count();
+    return QPair<int, int> (count, hiddenCount);
 #else
     // Taken from kdelibs/kio/kio/kdirmodel.cpp
     // Copyright (C) 2006 David Faure <faure@kde.org>
 
     int count = -1;
+    int hiddenCount = -1;
     DIR* dir = ::opendir(QFile::encodeName(path).constData());
     if (dir) {
         count = 0;
+        hiddenCount = 0;
         struct dirent *dirEntry = nullptr;
         while ((dirEntry = ::readdir(dir))) { // krazy:exclude=syscalls
             if (dirEntry->d_name[0] == '.') {
@@ -510,12 +546,15 @@ int FileMetaDataProvider::subDirectoriesCount(const QString& path)
                     // Skip ".."
                     continue;
                 }
+                // hidden files
+                hiddenCount++;
+            } else {
+                ++count;
             }
-            ++count;
         }
         ::closedir(dir);
     }
-    return count;
+    return QPair<int, int>(count, hiddenCount);
 #endif
 }
 
