@@ -26,9 +26,13 @@
 #include "widgetfactory.h"
 #include "filemetadataprovider.h"
 
+#include <KConfig>
+#include <KConfigGroup>
+
 #include <KFileItem>
 #include <KFileMetaData/UserMetaData>
 
+#include <QCheckBox>
 #include <QGridLayout>
 #include <QLabel>
 #include <QSpacerItem>
@@ -46,6 +50,7 @@ class Baloo::FileMetaDataWidget::Private
 public:
     struct Row
     {
+        QCheckBox* checkBox;
         QLabel* label;
         QWidget* value;
     };
@@ -53,13 +58,6 @@ public:
     Private(FileMetaDataWidget* parent);
     ~Private();
 
-    /**
-     * Parses the configuration file "kmetainformationrc" and
-     * updates the visibility of all rows that got their data
-     * from KFileItem.
-     */
-    void updateFileItemRowsVisibility();
-
     void deleteRows();
 
     void slotLoadingFinished();
@@ -70,6 +68,8 @@ public:
     QStringList sortedKeys(const QVariantMap& data) const;
     QLabel* createLabel(const QString &key,  const QString& itemLabel, FileMetaDataWidget* parent);
 
+    void saveConfig();
+
     QList<Row> m_rows;
     FileMetaDataProvider* m_provider;
     QGridLayout* m_gridLayout;
@@ -77,6 +77,9 @@ public:
     MetadataFilter* m_filter;
     WidgetFactory* m_widgetFactory;
 
+    QMap<QString, bool> m_visibilityChanged;
+    bool m_configureVisibleProperties = false;
+
 private:
     FileMetaDataWidget* const q;
 };
@@ -95,7 +98,7 @@ FileMetaDataWidget::Private::Private(FileMetaDataWidget* parent)
     // TODO: If KFileMetaDataProvider might get a public class in future KDE releases,
     // the following code should be moved into KFileMetaDataWidget::setModel():
     m_provider = new FileMetaDataProvider(q);
-    connect(m_provider, SIGNAL(loadingFinished()), q, SLOT(slotLoadingFinished()));
+    connect(m_provider, &FileMetaDataProvider::loadingFinished, q, [this](){ slotLoadingFinished(); });
 }
 
 FileMetaDataWidget::Private::~Private()
@@ -104,17 +107,21 @@ FileMetaDataWidget::Private::~Private()
 
 void FileMetaDataWidget::Private::deleteRows()
 {
-    foreach (const Row& row, m_rows) {
+    for (const Row& row : qAsConst(m_rows)) {
         delete row.label;
         row.value->deleteLater();
+        if (row.checkBox) {
+            row.checkBox->deleteLater();
+        }
     }
 
     m_rows.clear();
 }
 
-QLabel* FileMetaDataWidget::Private::createLabel(const QString &key, const QString& itemLabel, FileMetaDataWidget* parent) 
+QLabel* FileMetaDataWidget::Private::createLabel(const QString &key, const QString& itemLabel, FileMetaDataWidget* parent)
 {
     QLabel* label = new QLabel(itemLabel + QLatin1Char(':'), parent);
+    label->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::TextSelectableByKeyboard);
     label->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
     label->setForegroundRole(parent->foregroundRole());
     label->setFont(parent->font());
@@ -127,36 +134,83 @@ QLabel* FileMetaDataWidget::Private::createLabel(const QString &key, const QStri
 void FileMetaDataWidget::Private::slotLoadingFinished()
 {
     deleteRows();
-    
+
     if (m_gridLayout == nullptr) {
         m_gridLayout = new QGridLayout(q);
         m_gridLayout->setContentsMargins(0, 0, 0, 0);
         m_gridLayout->setSpacing(q->fontMetrics().height() / 4);
-    } 
+    }
 
-    QVariantMap data = m_filter->filter( m_provider->data() );
-    m_widgetFactory->setNoLinks( m_provider->realTimeIndexing() );
+    QVariantMap data = m_provider->data();
+    QStringList active;
+    if (m_configureVisibleProperties) {
+        active = m_filter->filter(data).keys();
+        auto changedIt = m_visibilityChanged.constBegin();
+        while (changedIt != m_visibilityChanged.constEnd()) {
+            if (changedIt.value()) {
+                active.append(changedIt.key());
+            } else {
+                active.removeAll(changedIt.key());
+            }
+            changedIt++;
+        }
+        m_widgetFactory->setReadOnly(true);
+        m_gridLayout->setColumnStretch(0, 1);
+        m_gridLayout->setColumnStretch(1, 3);
+        m_gridLayout->setColumnStretch(2, 0);
+        m_gridLayout->setColumnStretch(3, 6);
+    } else {
+        data = m_filter->filter(data);
+        m_widgetFactory->setReadOnly(m_provider->isReadOnly());
+        m_gridLayout->setColumnStretch(0, 4);
+        m_gridLayout->setColumnStretch(1, 0);
+        m_gridLayout->setColumnStretch(2, 6);
+        m_gridLayout->setColumnStretch(3, 0);
+    }
 
     int rowIndex = 0;
     // Iterate through all remaining items.
     // Embed the label and the value as new row in the widget
     const QStringList keys = sortedKeys(data);
-    for (const auto key: keys) {
+    const int spacerWidth = QFontMetrics(q->font()).size(Qt::TextSingleLine, QStringLiteral(" ")).width();
+
+    const int labelColumn = m_configureVisibleProperties ? 1 : 0;
+
+    for (const auto& key : keys) {
         Row row;
-        const int spacerWidth = QFontMetrics(q->font()).size(Qt::TextSingleLine, " ").width();
-        m_gridLayout->addItem(new QSpacerItem(spacerWidth, 1), rowIndex, 1);
+        if (m_configureVisibleProperties) {
+            row.checkBox = new QCheckBox(q);
+            if (active.contains(key)) {
+                row.checkBox->setChecked(true);
+            }
+            m_gridLayout->addWidget(row.checkBox, rowIndex, 0, Qt::AlignTop | Qt::AlignRight);
+            connect(row.checkBox, &QCheckBox::stateChanged,
+                    q, [this, key](int state) { this->m_visibilityChanged[key] = (state == Qt::Checked); });
+        } else {
+            row.checkBox = nullptr;
+        }
 
         row.label = createLabel(key, m_provider->label(key), q);
-        m_gridLayout->addWidget(row.label, rowIndex, 0, Qt::AlignRight);
+        m_gridLayout->addWidget(row.label, rowIndex, labelColumn + 0, Qt::AlignRight);
+
+        m_gridLayout->addItem(new QSpacerItem(spacerWidth, 1), rowIndex, labelColumn + 1);
 
         row.value = m_widgetFactory->createWidget(key, data[key], q);
-        m_gridLayout->addWidget(row.value, rowIndex, 2, Qt::AlignLeft);
+        m_gridLayout->addWidget(row.value, rowIndex, labelColumn + 2, Qt::AlignLeft);
+
+        m_gridLayout->setRowStretch(rowIndex, 0);
 
         // Remember the label and value-widget as row
         m_rows.append(row);
         ++rowIndex;
     }
 
+    // Add vertical stretch - when the widget is embedded with extra vertical
+    // space, it should be added at the bottom, not distributed between the
+    // items.
+    m_gridLayout->addItem(new QSpacerItem(0, 0), rowIndex, 0, 1, -1);
+    m_gridLayout->setRowStretch(rowIndex, 1);
+
     q->updateGeometry();
     emit q->metaDataRequestFinished(m_provider->items());
 }
@@ -209,6 +263,24 @@ QStringList FileMetaDataWidget::Private::sortedKeys(const QVariantMap& data) con
     return list;
 }
 
+void FileMetaDataWidget::Private::saveConfig()
+{
+    if (m_visibilityChanged.isEmpty()) {
+        return;
+    }
+
+    KConfig config(QStringLiteral("baloofileinformationrc"), KConfig::NoGlobals);
+    KConfigGroup showGroup = config.group("Show");
+
+    auto changedIt = m_visibilityChanged.constBegin();
+    while (changedIt != m_visibilityChanged.constEnd()) {
+        showGroup.writeEntry(changedIt.key(), changedIt.value());
+        changedIt++;
+    }
+
+    showGroup.sync();
+}
+
 FileMetaDataWidget::FileMetaDataWidget(QWidget* parent)
     : QWidget(parent)
     , d(new Private(this))
@@ -222,28 +294,8 @@ FileMetaDataWidget::~FileMetaDataWidget()
 
 void FileMetaDataWidget::setItems(const KFileItemList& items)
 {
-    KFileItemList localItemsList;
-    QStringList list;
-
-    bool xAttrSuppored = true;
-
-    foreach(const KFileItem& item, items) {
-        QUrl url = item.targetUrl();
-        if (url.isLocalFile()) {
-            localItemsList << item;
-            QString path = url.toLocalFile();
-            list << path;
-
-            KFileMetaData::UserMetaData md(path);
-            xAttrSuppored &= md.isSupported();
-        }
-    }
-    setReadOnly(!xAttrSuppored);
-
-    d->m_provider->setItems(localItemsList);
-    d->m_widgetFactory->setItems(list);
-
-    setReadOnly(!xAttrSuppored);
+    d->m_provider->setItems(items);
+    d->m_widgetFactory->setItems(items);
 }
 
 KFileItemList FileMetaDataWidget::items() const
@@ -281,7 +333,7 @@ QSize FileMetaDataWidget::sizeHint() const
     int leftWidthMax = 0;
     int rightWidthMax = 0;
     int rightWidthAverage = 0;
-    foreach (const Private::Row& row, d->m_rows) {
+    for (const Private::Row& row : qAsConst(d->m_rows)) {
         const QWidget* valueWidget = row.value;
         const int rightWidth = valueWidget->sizeHint().width();
         rightWidthAverage += rightWidth;
@@ -307,7 +359,7 @@ QSize FileMetaDataWidget::sizeHint() const
 
     // Based on the available width calculate the required height
     int height = d->m_gridLayout->margin() * 2 + d->m_gridLayout->spacing() * (d->m_rows.count() - 1);
-    foreach (const Private::Row& row, d->m_rows) {
+    for (const Private::Row& row : qAsConst(d->m_rows)) {
         const QWidget* valueWidget = row.value;
         const int rowHeight = qMax(row.label->heightForWidth(leftWidthMax),
                                    valueWidget->heightForWidth(rightWidthMax));
@@ -320,4 +372,18 @@ QSize FileMetaDataWidget::sizeHint() const
     return QSize(width, height);
 }
 
+void FileMetaDataWidget::setConfigurationMode(ConfigurationMode mode)
+{
+    if (mode == ConfigurationMode::ReStart) {
+        d->m_configureVisibleProperties = true;
+    } else if (mode == ConfigurationMode::Accept) {
+        d->saveConfig();
+        d->m_configureVisibleProperties = false;
+    } else if (mode == ConfigurationMode::Cancel) {
+        d->m_configureVisibleProperties = false;
+    }
+    d->m_visibilityChanged.clear();
+    d->slotLoadingFinished();
+}
+
 #include "moc_filemetadatawidget.cpp"
